/*
 * Ad.c
 *
 * AD Converter
 *
 * ATmega88/328
 *
 * Created: 2011/12/31 7:59:21
 *  Author: Satoshi
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>

volatile static double  AdcAccumulator;
volatile static short   AdcSampleCounter;
volatile static short   AdcTemp;
volatile static uint8_t AdcResult;
volatile static uint8_t AdcNo = -1;

#define ADCOFFSET	132121			// Offset error compensation, (n * 4096)
#define ADCGAIN		0.972970869		// Gain error compensation

ISR(ADC_vect)
{
	AdcAccumulator += ADC;
	if (AdcSampleCounter++ > 4096-1) {
		ADCSRA &= ~_BV(ADIE);				// ADC Interrupt Dis enable
		AdcResult = 1;
	}
	else ADCSRA |= _BV(ADSC);				// ADC Start
}


void AdcSelect( uint8_t no )
{
											// Select to Multiplexer
	if (AdcNo != no) {
		AdcNo = 0x1f & no;
		ADMUX = (1<<REFS1 | 1<<REFS0) | AdcNo;	// REFS1, REFS0:11: Internal Ref 1.1V
		DIDR0 = AdcNo;							// Dis enable of Input buffer
	}	
}

void AdcStart( void )
{
											// ADC Enable, ADC Interrupt Enable, ADC Start
	AdcResult = 0;
	ADCSRA |= (1<<ADEN | 1<<ADIE | 1<<ADSC );
}

int8_t AdcVin( double *Vin )
{
	int16_t  Offset;
	double   Gain;
	
	if (AdcResult) {
		AdcAccumulator += ADCOFFSET;				// Offset error compensation, (n*4096)/64
		AdcAccumulator *= ADCGAIN;					// Gain error compensation
		AdcTemp=(int)AdcAccumulator%64;
		AdcAccumulator/=64;
		if(AdcTemp>=32) AdcAccumulator += 1;	// Rounding up
	 	*Vin = (AdcAccumulator/65536)*1.100;	// VREF = 1.100;
		AdcSampleCounter    = 0;			
		AdcAccumulator		= 0;
				
		return 0;	
	}
	else return -1;
}

void AdcInitialize(void)
{
											// ADEN:ADC Enable:0  ADIE:ADC Interrupt Enable:0
											// ADSC:ADC Start Conversion:0 ADATE:ADC Auto Trigger Enable
											// ADPS0,S1,S2:ADC PreScaler:111: 1/128
											// ADPS0,S1,S2:ADC PreScaler:110: 1/ 64
	ADCSRA = (0<<ADEN)|(0<<ADIE)|(0<<ADSC)|(0<<ADATE)|(1<<ADPS2)|(1<<ADPS1)|(0<<ADPS0);
	AdcSelect( 0 );
}
