/*
 * BIOTOPE_1.c
 *
 * Control of Biotope 1 Ver. 1.1.7(2013.01.04)
 *
 * CPU : ATmega328P (ROM 32Kbyte RAM 2Kbyte EEPROM 1024byte 8MHz) 
 *
 * Optimization Level : optimize(-O1)
 *
 * Created: 2011/12/11 10:22:29
 *  Author: Satoshi
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include <stdlib.h>


#include "TWI_Master.h"
#include "Uart.h"
#include "ADC.h"
#include "RTC4543.h"

								//LED control
#define	LED_OFF			0			//LED Off
#define	LED_ON			1			//LED On
#define LED_VERYSLOW	2			//LER blink of very slow
#define	LED_SLOW		3			//LED blink of slow
#define	LED_NORMAL		4			//LED blink of normal
#define	LED_HI			5			//LED blink of hi

								//ADC
#define ADC_0_AIR		0			//AIR
#define ADC_1_NO1		1			//Box 1
#define ADC_2_NO2		2			//Box 2
#define ADC_3_NO3		3			//Box 3
#define ADC_8_CPU		8			//CPU

								//Run mode
#define MODE_AUTO		0			//Auto
#define MODE_ON			1			//On
#define MODE_OFF		2			//Off

								//Operation
#define SELECT_NORMAL		0		//Normal
#define SELECT_HI			1		//Hi temp.
#define SELECT_LOW			2		//Low temp.
#define SELECT_HI_RESET		3		//Rest of Hi temp.
#define SELECT_LOW_RESET	4		//Rest of Low temp.
#define SELECT_DAY_0		5		//Log of today
#define SELECT_DATE			6		//Set of date
#define SELECT_TIME			7		//Set of time 
#define SELECT_TIME_ADJUST	8		//Adjust of time
#define SELECT_DEBUG		9		//Debug

								//Control of AC Line
#define AC_ON		1				//On
#define AC_OFF		0				//Off

								//Sensor of Temperature
#define TEMP_AIR	0				//Air temperature (ADC0)
#define TEMP_UNIT1	1				//Unit 1 temperature (ADC1)
#define TEMP_UNIT2	2				//Unit 2 temperature (ADC2)
#define TEMP_UNIT3	3				//Unit 3 temperature (ADC3)
#define TEMP_CPU	4				//Controller temperature (CPU)

								//LED control of LCD Unit
#define	LCD_LED_OFF	0				//LED off 
#define	LCD_LED_ON	1				//LED On

								//Alarm of Water level
#define ALARM_OFF	0				//Normal
#define ALARM_ON	1				//Alarm

#define TEMP_TABLE_MAX 6

#define TEMP_MAX	5			//Max of temperature sensor
#define PEAK_MAX	3			//Max of Peak data

								//Port B
#define	LED			0			//W : LED
#define	PORTB_1		1			//RW: RTC DATA
#define	PORTB_2		2			//W : RTC CLK
#define	WATER_LEVEL	3			//R : Level of water level (SPI MOSI)
#define	PORTB_4		4			//  : (SPI MISO)
#define	PORTB_5		5			//  : (SPI SCK)
#define	PORTB_6		6			//W : RTC WR
#define	PORTB_7		7			//W : RTC CD

								//Port D
#define PORTD_0		0			//R : (UART RTX)
#define PORTD_1		1			//  : (UART TXD)
#define	MODE_SW		2			//R : Mode   SW
#define	SELECT_SW	3			//R : Select SW
#define	ENTER_SW	4			//R : Enter  SW 
#define PLUS_SW		5			//R : PLUS   SW
#define MINUS_SW	6			//R : MINUS  SW
#define AC			7			//W : AC OUT (Pump)




#define TWI_GEN_CALL         0x00  // The General Call address is 0

// TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20

// TWI transmission states, used in the main application.
#define SEND_DATA             0x01
#define REQUEST_DATA          0x02
#define READ_DATA_FROM_BUFFER 0x03

unsigned char TWI_Act_On_Failure_In_Last_Transmission ( unsigned char TWIerrorMsg ) {
	// A failure has occurred, use TWIerrorMsg to determine the nature of the failure
    // and take appropriate actions.
    // Se header file for a list of possible failures messages.
     
    // Here is a simple sample, where if received a NACK on the slave address,
    // then a retransmission will be initiated.
 
	if ( (TWIerrorMsg == TWI_MTX_ADR_NACK) | (TWIerrorMsg == TWI_MRX_ADR_NACK) )
	    TWI_Start_Transceiver();
    
	return TWIerrorMsg; 
}

volatile static uint32_t	ucnt    = 0;			//Time counter
volatile static uint8_t		adjust1 = 0;			//Time Adjustment counter 1
volatile static uint8_t		adjust2 = 0;			//Time Adjustment counter 2
volatile static uint8_t		adjust3 = 0;			//Time Adjustment counter 3

volatile static unsigned char TWI_operation = 0;

static uint8_t	led_sts  = 0;						//LED 1:On 0:Off

static uint8_t	led_mode = 0;						//LED Mode
static uint8_t	led_cnt  = 0;						//LED Blink counter

static uint8_t	mode_sw_sts    = 0;					//Mode SW status
static uint8_t	select_sw_sts  = 0;					//Select SW status
static uint8_t	enter_sw_sts   = 0;					//Enter SW status
static uint8_t	plus_sw_sts    = 0;					//Plus SW status
static uint8_t	minus_sw_sts   = 0;					//Minus SW status

static uint8_t	usar_mode = 1;						//USAR Output 1:on 0:off

static int8_t   temp_current = 0;
static int16_t	temp_c[TEMP_MAX];					//Temp data xxx (temp*10)
static uint8_t	peak_index[PEAK_MAX];				//Peak date
static int16_t  peak_h[PEAK_MAX][TEMP_MAX];			//Temp data xxx (temp*10)
static int16_t  peak_l[PEAK_MAX][TEMP_MAX];			//Temp data xxx (temp*10)
static int16_t	temp_day[24][TEMP_MAX];				//Temp data xxx (temp*10) 24h log

static struct DATE date;

static uint8_t	lcd_led_mode = -1;
 
const int16_t   temp_table[] = {50,100,150,200,250,300,800};				//Temp * 10
const uint16_t  time_on_table[]  = {   10, 4800,4800,4800,4800,4800,4800};	//Sec * 10
const uint16_t  time_off_table[] = {10200,10200,9000,6600,4800,3000,1200};	//Sec * 10
const int8_t	temp_hysteresis = 10;										//Temp hysteresis data
const double    temp_ajdsut_offset[] ={-2.8816, -3.2667, -3.3333, -3.3861 };
const double    temp_adjsut_gain[]   ={1.0, 1.0, 1.0, 1.0 };

const uint8_t   measurement[] = {ADC_0_AIR, ADC_1_NO1, ADC_2_NO2, ADC_3_NO3, ADC_8_CPU};

const char mode_table[][2]   = {"A","O","X"};
const char select_table[][5] = {"    "," HI "," LOW"," HRS"," LRS","Tm","DAT"," TIM"," AJT"," DUB"};

void Led_On ( void )								//LED On
{
	PORTB |= _BV(LED);
	led_sts = LED_ON;
}

void Led_Off ( void )								//LED Off
{
	PORTB &= ~_BV(LED);
	led_sts = LED_OFF;
}

void Led_Blink ( void )								//LED Blink
{
	if (led_sts != LED_OFF ) Led_Off();
	else                     Led_On();
}


void LedModeSet( uint8_t mode )						//LED Set of mode
{
	led_mode = mode;
}

void LedControl(void)								//LED control (CTC Interrupt)
{
	led_cnt++;
	switch(	led_mode ) {							//Interval time 0.1mSec
	case LED_OFF:
		Led_Off();
		break;
	case LED_ON:
		Led_On();
		break;
	case LED_VERYSLOW:
		if ( led_cnt > 15 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_SLOW:
		if ( led_cnt > 10 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_NORMAL:
		if ( led_cnt > 5 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_HI:
		if ( led_cnt > 1 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	}
}

													//Timer 0: Time control
ISR ( TIMER0_COMPA_vect )							//Interrupt Timer 0 CTC mode Clock 1MHz
{
	if ( ++adjust3 >= 4 ) {							//0.0256 Sec  Clock 8MHz Only
		ucnt++;										//0.1024 Sec
		if ( ++adjust1 >= 50 ) {					//Adjust of 0.002Sec
			ucnt++;
			adjust1 = 0;
		}	
		if (++adjust2 >= 250 ) {					//Adjust of 0.0004Sec
			ucnt++;
			adjust2 = 0;
		}
		LedControl();
		adjust3 = 0;
	}
}

void TimeInit( void )								//Initialize of Timer 0
{
	TCCR0B = 0b00000000;							//Stop to Timer 
	TCCR0A = 0b00000010;							//mode is CTC
	OCR0A  = 200;									//1MHz 100  8MHz 200 (Clock 1MHz 0.1024Sec 8MHz 0.0256Sec)
	TCCR0B = 0b00000101;							//clock 1/1024 

	TIMSK0  = _BV(OCIE0A);							//Interrupt of comp match A

	ucnt   = 0;										//stick counter
	adjust1 = 0;									//adjustment counter 1
	adjust2 = 0;									//adjustment counter 2
	adjust3 = 0;									//adjustment counter 3
}

uint8_t TimeCheck( uint32_t start, uint32_t time )	//Timer check
{
	uint8_t sts = 0;

	if (ucnt >= start) {
		if (time >= start) {
			if (ucnt >= time) sts = 1;
		}
	}
	else {
		if (time >= start)    sts = 1;
		else {
			if (ucnt >= time) sts = 1;
		}
	} 
	return sts;
}

void TimeSet( uint32_t *start, uint32_t *time )		//Timer set
{
	*start = ucnt;
	*time  = *start + *time;
}

int TimeGet( uint8_t mode )
{
	uint8_t i;
	
	for (i=0; i<=TEMP_TABLE_MAX; i++) {
		if (temp_c[TEMP_UNIT1] <= temp_table[i]) break;
	}
	
	if (temp_current >= 0) {
		if (temp_current != i) {
			if ((temp_current < i) & (i != 0)) {
				if (temp_c[TEMP_AIR] >= temp_table[i-1] + temp_hysteresis) temp_current = i;
			}
			else temp_current = i;
		}
	}
	else temp_current = i;	

	if (mode) return time_on_table[temp_current];
	else      return time_off_table[temp_current];
}

void AC_Control ( uint8_t mode )					//AC Control
{	
	if (mode) PORTD |= _BV( AC );					//AC power on
	else      PORTD &= ~_BV( AC );					//AC power off
}


uint8_t ModeSw()									//Mode switch
{													//Read of mode switch
	uint8_t sts = 0;
	
	if (mode_sw_sts) {
		if (PIND & 1<<MODE_SW) mode_sw_sts = 0;
	}
	else {	
		if (PIND & 1<<MODE_SW) sts = 0;				//Active low
		else {
           sts = 1;
		   mode_sw_sts = 1;
		}		
	}
	return sts;
}


uint8_t	SelectSw()									//Select switch
{													//Read of Select switch
	uint8_t sts = 0;
	
	if (select_sw_sts) {
		if (PIND & 1<<SELECT_SW) select_sw_sts = 0;
	}
	else {	
		if (PIND & 1<<SELECT_SW) sts = 0;			//Active low
		else {
		   sts =1;
		   select_sw_sts = 1;
		}
	}
	return sts;			
}

uint8_t	EnterSw()									//Enter switch
{													//Read of Enter switch
	uint8_t sts = 0;
	
	if (enter_sw_sts) {
		if (PIND & 1<<ENTER_SW) enter_sw_sts = 0;
	}
	else {	
		if (PIND & 1<<ENTER_SW) sts = 0;			//Active low
		else {
			sts = 1;
			enter_sw_sts = 1;
		}
	}
	return sts;			
}

uint8_t	PlusSw()									//Plus switch
{
	uint8_t sts = 0;
	
	if (plus_sw_sts) {
		if (PIND & 1<<PLUS_SW) plus_sw_sts = 0;
	}
	else {
		if (PIND & 1<<PLUS_SW) sts = 0;				//Active low
		else {
			sts = 1;
			plus_sw_sts = 1;
		}		
	}
	return sts;	
}

uint8_t	MinusSw()									//Minus switch
{
	uint8_t sts = 0;
	
	if (minus_sw_sts) {
		if ( (PIND & 1<<MINUS_SW) ) minus_sw_sts = 0;
	}
	else {
		if (PIND & 1<<MINUS_SW) sts = 0;			//Active low
		else {
			sts = 1;
			minus_sw_sts = 1;
		}		
	}
	return sts;	
}

uint8_t WaterLevel()
{
	uint8_t sts = 0;
	
	if (PINB & (1<<WATER_LEVEL)) sts = 0;			//Active low
	else                         sts = 1;
	
	return sts;
}

void TwiOut( unsigned char id, char *data )
{
	unsigned char messageBuf[4];
	
	while( *data != 0x00 ) {
		messageBuf[0] = (id<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);	// The first byte must always consit of General Call code or the TWI slave address.
		messageBuf[1] = TWI_CMD_MASTER_WRITE;						// The first byte is used for commands.
		messageBuf[2] = *data++;									// The second byte is used for the data.
		TWI_Start_Transceiver_With_Data( messageBuf, 3 );
	}
}

void PrintOut( unsigned char id, char *data )
{	
	if (usar_mode) UartSendString( data );
	TwiOut( id, data );
}


void LocateDisplay( unsigned char TWI_targetSlaveAddress, uint8_t y, uint8_t x ) {
	char pbuff[8];
	
	pbuff[0] = 0x1b;
	pbuff[1] = '[';
	pbuff[2] = '0' + y;
	pbuff[3] = ';';
	if (x < 10) {
		pbuff[4] = '0' + x;
		pbuff[5] = 'H';
		pbuff[6] = NULL;
	}
	else {
		pbuff[4] = '0' + x / 10;
		pbuff[5] = '0' + x % 10;
		pbuff[6] = 'H';
		pbuff[7] = NULL;
	}
	if (usar_mode) 	UartSendString( pbuff );
	TwiOut( TWI_targetSlaveAddress, pbuff );	
}

void LocateClearDisplay( unsigned char TWI_targetSlaveAddress, uint8_t y, uint8_t x ) {

	LocateDisplay( TWI_targetSlaveAddress, y, x );
	PrintOut( TWI_targetSlaveAddress, "     " );
	LocateDisplay( TWI_targetSlaveAddress, y, x );
}

void LcdLedControl( unsigned char TWI_targetSlaveAddress, uint8_t mode )
{
	
	if (mode == LCD_LED_ON) {
		if (lcd_led_mode != LCD_LED_ON) {
			PrintOut( TWI_targetSlaveAddress, "\x11");
			lcd_led_mode = LCD_LED_ON;
		}		
	}	
	else {
		if (lcd_led_mode != LCD_LED_OFF) {
			PrintOut( TWI_targetSlaveAddress, "\x12");
			lcd_led_mode = LCD_LED_OFF;
		}		
	}	
}

TWI_Check( unsigned char TWI_targetSlaveAddress )
{
	unsigned char messageBuf[4], cmd;
	if ( ! TWI_Transceiver_Busy() ) {
																	// Check if the last operation was successful
		if ( TWI_statusReg.lastTransOK ) {
			if ( TWI_operation ) {									// Section for follow-up operations.
																	// Determine what action to take now
				if (TWI_operation == REQUEST_DATA) {				// Request/collect the data from the Slave
					messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
					TWI_Start_Transceiver_With_Data( messageBuf, 2 );       
					TWI_operation = READ_DATA_FROM_BUFFER;			// Set next operation        
				}
				else if (TWI_operation == READ_DATA_FROM_BUFFER) {	// Get the received data from the transceiver buffer
					TWI_Get_Data_From_Transceiver( messageBuf, 2 );
					TWI_operation = FALSE;							// Set next operation        
				}
			}
		}
		else  {														// Got an error during the last transmission
																	// Use TWI status information to detemine cause of failure and take appropriate actions. 
			TWI_Act_On_Failure_In_Last_Transmission( TWI_Get_State_Info( ) );
		}
	}
}

int main(void)
{
	unsigned char TWI_targetSlaveAddress;
	unsigned char cmd;
	uint32_t start, time, wait_start, wait_time, date_start, date_time;
	uint8_t ac_out, ac_out_save;
	uint8_t mode, mode_old;
	uint8_t select, select_old;
	uint8_t mcnt, m_ready = 0;
	uint8_t dsp_init = 0;
	uint8_t date_init= 0, date_cnt, date_disp;
	uint8_t select_init = 0;
	uint8_t alarm = 0;
	int16_t i, j, k;
	int8_t peak_cnt, peak_no, peak_dsp, hour_dsp, hour_no, adjust_no;
	uint8_t day_current, hour_current;
	
	char pbuff[8];
	
	double  Vin;
	int16_t Vin10;

	// Initialize of port
	DDRB  = 0b11000101;
	PORTB = 0b00001010;
    DDRD  = 0b10000000;
	PORTD = 0b01111100;
		
	TWI_Master_Initialise();

	//  Enable to Interrupt
	sei();
  
	TWI_targetSlaveAddress   = 0x20;		// LCD Controller

	UartInitialize( 9800 );					// Initialize Usart Port (RS232C)
	AdcInitialize();
	Rtc_Initial();

    TimeInit();
	wdt_enable(WDTO_500MS);
	
	start = time = 0;
	mode     = MODE_AUTO;
	mode_old = MODE_AUTO;
	
	select     = SELECT_NORMAL;
	select_old = SELECT_NORMAL;
	
	LedModeSet( LED_OFF );
	ac_out_save = AC_OFF;
	ac_out      = AC_OFF;
	AC_Control( ac_out );
	
	Rtc_Get( &date );
	day_current  = date.day;
	hour_current = date.hour;
    peak_cnt = 0;
	for (i=0; i<PEAK_MAX; i++) peak_index[i]     = 0;
	peak_index[peak_cnt]    = day_current;
	
	for (i=0; i<PEAK_MAX; i++) {
		for (j=0; j<TEMP_MAX; j++) {
			peak_h[i][j] = -51;
			peak_l[i][j] = 601;
		}		
	}
	for (j=0; j<24; j++) {
		for (k=0; k<TEMP_MAX; k++) {
			temp_day[j][k] = 0;
		}		
	}
	
	temp_current    = -1;
	mcnt = 0;

	UartSendString( "\n\ready." );
	
	for(;;) {
		alarm = WaterLevel();
		
		cmd = UartGetReceivedData();
		if (ModeSw() || cmd == 'A') {
			mode++;
			if (mode > MODE_OFF) mode = MODE_AUTO;
		}
		else if (SelectSw() || cmd == 'B') {
			select++;
			if (select > SELECT_DEBUG) select = SELECT_NORMAL;
		}
		switch( mode ) {
		case MODE_AUTO:
			if (mode_old != mode) {
				ac_out = ac_out_save;
				if (ac_out) LedModeSet( LED_NORMAL );
				else        LedModeSet( LED_SLOW );
				AC_Control( ac_out );
			}
			if ((start == 0) & (time == 0)) {
				time = TimeGet( AC_ON );
				ac_out = AC_ON;
				LedModeSet( LED_NORMAL );
				AC_Control( ac_out );
				TimeSet( &start, &time );
			}
			else {
				if (TimeCheck( start, time)) {
					if ( ac_out == AC_ON ) {
						time = TimeGet( AC_OFF );
						ac_out = AC_OFF;
						LedModeSet( LED_SLOW );
					}
					else {
						time = TimeGet( AC_ON );
						if (alarm == ALARM_OFF) {
							ac_out = AC_ON;
							LedModeSet( LED_NORMAL );
						}
						else {				
							ac_out = AC_OFF;
							LedModeSet( LED_SLOW );
						}						
					}
					AC_Control( ac_out );
					TimeSet( &start, &time );
				}
			}
			break;
		case MODE_ON:
			if (mode_old != mode) {
				ac_out_save = ac_out;
				ac_out = AC_ON;
				LedModeSet( LED_HI );
				AC_Control( ac_out );
			}
			break;
		case MODE_OFF:
			if (mode_old != mode) {
				ac_out = AC_OFF;
				LedModeSet( LED_VERYSLOW );
				AC_Control( ac_out );
			}			
			break;
		}
		if (mode_old != mode) {
				LocateDisplay(TWI_targetSlaveAddress, 2, 16);
				PrintOut( TWI_targetSlaveAddress, mode_table[mode] );
		}		
		mode_old = mode;
		switch( select ) {
		case SELECT_NORMAL:
			break;
		case SELECT_HI:
			if (select_init == 0) {
				peak_no     = peak_cnt;
				peak_dsp    = -1;
				select_init = 1;
			}
			if (PlusSw() || cmd == 'D') peak_no++;
			if (peak_no >= PEAK_MAX) peak_no = 0;
			if (MinusSw() || cmd == 'E') peak_no--;
			if (peak_no < 0) peak_no = PEAK_MAX - 1;
			if (peak_no != peak_dsp) {
				pbuff[0] = 'H';
				pbuff[1] = 'i';
				pbuff[2] = '0' + peak_index[peak_no] / 10;
				pbuff[3] = '0' + peak_index[peak_no] % 10;
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				peak_dsp = peak_no;
			}			
			break;
		case SELECT_LOW:
			if (select_init == 0) {
				peak_no     = peak_cnt;
				peak_dsp    = -1;
				select_init = 1;
			}
			if (PlusSw() || cmd == 'D') peak_no++;
			if (peak_no >= PEAK_MAX) peak_no = 0;
			if (MinusSw() || cmd == 'E') peak_no--;
			if (peak_no < 0) peak_no = PEAK_MAX - 1;
			if (peak_no != peak_dsp) {
				pbuff[0] = 'L';
				pbuff[1] = 'o';
				pbuff[2] = '0' + peak_index[peak_no] / 10;
				pbuff[3] = '0' + peak_index[peak_no] % 10;
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				peak_dsp = peak_no;
			}			
			break;
		case SELECT_HI_RESET:
			if (EnterSw() || (cmd =='C')) {
				for (i=0; i<TEMP_MAX; i++) peak_h[peak_cnt][i] = -5.0;
				select = SELECT_NORMAL;
			}
			break;
		case SELECT_LOW_RESET:
			if (EnterSw() || (cmd =='C')) {
				for (i=0; i<TEMP_MAX; i++) peak_l[peak_cnt][i] = 60.0;
				select = SELECT_NORMAL;
			}
			break;
		case SELECT_DAY_0:
			if (select_init == 0) {
				hour_no     = 12;
				hour_dsp    = -1;
				select_init = 1;
			}
			if (PlusSw() || cmd == 'D') hour_no++;
			if (hour_no >= 24) hour_no = 0;
			if (MinusSw() || cmd == 'E') hour_no--;
			if (hour_no < 0) hour_no = 23;
			if (hour_no != hour_dsp) {
				pbuff[0] = 'T';
				pbuff[1] = 'm';
				pbuff[2] = '0' + hour_no / 10;
				pbuff[3] = '0' + hour_no % 10;
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				hour_dsp = hour_no;
			}			
			break;
		case SELECT_DATE:
			if (select_init == 0) {
				date_cnt    = 0;
				select_init = 1;
			}
			switch( date_cnt ) {
			case 0:
				pbuff[0] = 'Y';
				pbuff[1] = '0' + date.year / 10;
				pbuff[2] = '0' + date.year % 10;
				pbuff[3] = ' ';
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				if (PlusSw()  || cmd == 'D') date.year++;
				if (date.year > 99) date.year = 12;
				if (MinusSw() || cmd == 'E') date.year--;
				if (date.year < 12) date.year = 99;
				break;
			case 1:
				pbuff[0] = 'M';
				pbuff[1] = '0' + date.mouth / 10;
				pbuff[2] = '0' + date.mouth % 10;
				pbuff[3] = ' ';
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				if (PlusSw()  || cmd == 'D') date.mouth++;
				if (date.mouth > 12) date.mouth = 1;
				if (MinusSw() || cmd == 'E') date.mouth--;
				if (date.mouth < 1 ) date.mouth = 12;
				break;
			case 2:
				pbuff[0] = 'D';
				pbuff[1] = '0' + date.day / 10;
				pbuff[2] = '0' + date.day % 10;
				pbuff[3] = ' ';
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				if (PlusSw()  || cmd == 'D') date.day++;
				if (date.day > 31) date.day = 1;
				if (MinusSw() || cmd == 'E') date.day--;
				if (date.day < 1 ) date.day = 31;
				break;
			}
			if (EnterSw() || cmd == 'C') {
				if (++date_cnt > 2) {
					Rtc_Put( &date );
					select = SELECT_NORMAL;
				}				
			}
			break;
		case SELECT_TIME:
			if (select_init == 0) {
				date_cnt    = 0;
				select_init = 1;
			}
			switch( date_cnt ) {
			case 0:
				pbuff[0] = 'h';
				pbuff[1] = '0' + date.hour / 10;
				pbuff[2] = '0' + date.hour % 10;
				pbuff[3] = ' ';
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				if (PlusSw()  || cmd == 'D') date.hour++;
				if (date.hour > 23) date.hour = 0;
				if (MinusSw() || cmd == 'E') date.hour--;
				if (date.hour > 23) date.hour = 23;				//UINT
				break;
			case 1:
				pbuff[0] = 'm';
				pbuff[1] = '0' + date.minute / 10;
				pbuff[2] = '0' + date.minute % 10;
				pbuff[3] = ' ';
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				if (PlusSw()  || cmd == 'D') date.minute++;
				if (date.minute > 59) date.minute = 0;
				if (MinusSw() || cmd == 'E') date.minute--;
				if (date.minute > 59 ) date.minute = 59;		//UINT
				break;
			}			
			if (EnterSw() || cmd == 'C') {
				if (++date_cnt > 1) {
					Rtc_Put( &date );
					select = SELECT_NORMAL;
				}				
			}	
			break;
		case SELECT_TIME_ADJUST:
			Rtc_Get( &date );
			if (select_init == 0) {
				adjust_no   = date.second;
				select_init = 1;
			}
			if (date.second != adjust_no) {
				pbuff[0] = 'A';
				pbuff[1] = '0' + date.second / 10;
				pbuff[2] = '0' + date.second % 10;
				pbuff[3] = ' ';
				pbuff[4] = NULL;
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, pbuff );
				adjust_no = date.second;
			}			
			if (EnterSw() || cmd == 'C') {
				if (date.second >= 30) {
					date.second = 0;
					date.minute++;
					if (date.minute > 59) {
						date.minute = 0;
						date.hour++;
						if (date.hour >= 24) date.hour = 0;
					}

				}
				else date.second = 0;
				Rtc_Put( &date );
				select = SELECT_NORMAL;
			}	
			break;
		case SELECT_DEBUG:
			if (EnterSw() || (cmd =='C')) {
				usar_mode ^= 1;
				select = SELECT_NORMAL;
			}

			break;
		}

		if (dsp_init == 1) {
			if (select_old != select) {
				LocateDisplay(TWI_targetSlaveAddress, 1, 13);
				PrintOut( TWI_targetSlaveAddress, select_table[select]);
				select_init = 0;
			}
		}		
		select_old = select;
		
		if (select == SELECT_NORMAL ) {
			
			if ( date_init == 0 ) {
				date_time = 10;
				TimeSet( &date_start, &date_time);
				date_init = 1;
				date_disp = 0;
			}
			else {
				if (TimeCheck( date_start, date_time )) {
					Rtc_Get( &date );
					if (date_disp >= 3) {
						pbuff[0] = '0' + date.mouth / 10;
						pbuff[1] = '0' + date.mouth % 10;
						pbuff[2] = '0' + date.day   / 10;
						pbuff[3] = '0' + date.day   % 10;
						pbuff[4] = NULL;
						if (alarm == ALARM_ON) LcdLedControl(TWI_targetSlaveAddress, LCD_LED_OFF);
						date_disp = 0;
					}
					else {
						pbuff[0] = '0' + date.hour / 10;
						pbuff[1] = '0' + date.hour % 10;
						pbuff[2] = '0' + date.minute / 10;
						pbuff[3] = '0' + date.minute % 10;
						pbuff[4] = NULL;
						if (alarm == ALARM_ON) LcdLedControl(TWI_targetSlaveAddress, LCD_LED_ON);
					}
					date_disp++;
					LocateDisplay(TWI_targetSlaveAddress, 1, 13);
					PrintOut( TWI_targetSlaveAddress, pbuff);
					
					date_time = 10;
					TimeSet( &date_start, &date_time);
					
					if (alarm == ALARM_OFF) {
						if (date.hour >= 18) LcdLedControl( TWI_targetSlaveAddress, LCD_LED_ON );
						else                 LcdLedControl( TWI_targetSlaveAddress, LCD_LED_OFF);
					}					

				}
			}
		}
		
		TWI_Check(TWI_targetSlaveAddress);
		
//		cmd = UartGetReceivedData();

		switch ( m_ready ) {
		case 0:	
			AdcSelect( measurement[mcnt] );
			wait_time = 1;
			TimeSet( &wait_start, &wait_time );
			m_ready = 1;
			break;
		case 1:	
			if (TimeCheck( wait_start, wait_time)) m_ready = 2;
			break;
		case 2:
			AdcStart();
			m_ready = 3;
			break;
		case 3:
			if ( AdcVin( &Vin) >= 0 ) {
				if (select != SELECT_DEBUG) {
					if (mcnt != 4) Vin = ((Vin - 0.6) * 100 + temp_ajdsut_offset[mcnt]) * temp_adjsut_gain[mcnt];
					else           Vin = Vin * 772.983165992823 - 273.15 - 16.207;
					
					Vin10 = Vin * 10;
					temp_c[mcnt] = Vin10;
					if (peak_h[peak_cnt][mcnt] < Vin10) peak_h[peak_cnt][mcnt] = Vin10;
					if (peak_l[peak_cnt][mcnt] > Vin10) peak_l[peak_cnt][mcnt] = Vin10;
					switch (select) {
					case SELECT_HI:
						Vin = (double)peak_h[peak_no][mcnt] / 10.0 ;
						break;
					case SELECT_LOW:
						Vin = (double)peak_l[peak_no][mcnt] / 10.0;
						break;
					case SELECT_DAY_0:
						Vin = (double)temp_day[hour_no][mcnt] / 10.0;
						break;
					}								
				}

				if (dsp_init == 0) dsp_init = 1;
				switch (mcnt) {
				case 0:
					LocateClearDisplay(TWI_targetSlaveAddress, 1, 1);
					break;
				case 1:
				case 2:
				case 3:
					LocateClearDisplay(TWI_targetSlaveAddress, 2, (mcnt - 1) * 5 + 1);
					break;
				case 4:
					LocateClearDisplay(TWI_targetSlaveAddress, 1, 6);
					break;
				}
				if (select != SELECT_DEBUG) {
					FloatToStrF41( (float)Vin, pbuff );
				}					
				else {
					FloatToStrF51( (float)Vin * 100.0, pbuff );
				}
				
				PrintOut( TWI_targetSlaveAddress, pbuff );

				if (++mcnt >= TEMP_MAX) mcnt = 0;
				m_ready = 0;

				if (select == SELECT_NORMAL ) {				
					LocateDisplay(TWI_targetSlaveAddress, 1, 11);
					pbuff[0] = '0' + temp_current;
					pbuff[1] = NULL;
					PrintOut(TWI_targetSlaveAddress, pbuff);
				
					if (date.day != day_current) {
						if (++peak_cnt >= PEAK_MAX) peak_cnt = 0;
						day_current = date.day;
						peak_index[peak_cnt] = day_current;            
						for (i=0; i<TEMP_MAX; i++) {
							peak_h[peak_cnt][i] = -50;
							peak_l[peak_cnt][i] = 600;
						}
					}
					if (date.hour != hour_current) {
						hour_current = date.hour;
						for (i=0; i<TEMP_MAX; i++) {
							temp_day[hour_current][i] = temp_c[i];
						}
					}
				}
			}			
			break;
		}
			
		wdt_reset();		
	}
}

