/*
 * RTC4543.c
 *
 * Created: 2012/01/14 20:58:46
 *  Author: Satoshi
 */

 
#include "cpu_def.h"

#include <avr/io.h>
#include <util/delay.h>

#include "RTC4543.h"

#define RTC_CE		7
#define RTC_WR		6
#define RTC_CLK		2
#define RTC_DATA	1

static void Rtc_CLK()
{
	PORTB |= _BV(RTC_CLK);
	_delay_us(1);					//Delay 0.75 - 7800uSec
	PORTB &= ~_BV(RTC_CLK);
}

static void Rtc_CE( uint8_t mode )
{
	if (mode) PORTB |=  _BV( RTC_CE );
	else      PORTB &= ~_BV( RTC_CE );
}

static void Rtc_WR( uint8_t mode )
{
	if (mode) {
		PORTB |=  _BV( RTC_WR );
		DDRB  |=  _BV( RTC_DATA );			// Data Port : Out
		PORTB &= ~_BV( RTC_DATA );			// Data Port : Pull Up Off
	}	
	else {
	    PORTB &= ~_BV( RTC_WR );
		DDRB  &= ~_BV( RTC_DATA );			// Data Port : In
		PORTB |=  _BV( RTC_DATA );			// Data Port : Pull Up on
	}	
}

static uint8_t Rtc_GetBit()
{
	Rtc_CLK();
	if (PINB & (1 << RTC_DATA)) return 1;
	else                        return 0;
}

static uint8_t Rtc_GetBcd()
{
	uint8_t data, i;
	
	data = 0;
	for( i=0; i<=3; i++) data |= (Rtc_GetBit() << i);
	
	return data;
}

static void Rtc_PutBcd( uint8_t data)
{
	uint8_t i;
	
	for (i=0; i<=3; i++) {
		if (data & 0x01) PORTB |=  _BV(RTC_DATA);
		else             PORTB &= ~_BV(RTC_DATA);
		data = data >> 1;
		_delay_us(1);
		Rtc_CLK();
	}
}

void Rtc_Initial( void )
{
	DDRB  |= (1 << RTC_CLK) | (1 << RTC_WR) | (1 << RTC_CE);		// Out 
	PORTB |= (1 << RTC_WR);											// Pull Up
	Rtc_WR( 0 );
	Rtc_CE( 0 );
	PORTB &= ~_BV( RTC_CLK );
	PORTB &= ~_BV( RTC_DATA );
}

void Rtc_Get( struct DATE *date )
{
	Rtc_WR(0);
	Rtc_CE(1);
	_delay_us(1);
	date->second = Rtc_GetBcd() + (Rtc_GetBcd() & 0x07) * 10; 
	if (date->second >= 60) date->second = 59;
	date->minute = Rtc_GetBcd() + Rtc_GetBcd() * 10;
	if (date->minute >= 60) date->minute = 59;
	date->hour   = Rtc_GetBcd() + Rtc_GetBcd() * 10;
	if (date->hour >= 24)   date->hour   = 23;
	date->week   = Rtc_GetBcd();
	if (date->week >= 7)    date->week   = 6;
	date->day    = Rtc_GetBcd() + Rtc_GetBcd() * 10;
	if (date->day > 31)     date->day    = 31;
	date->mouth  = Rtc_GetBcd() + Rtc_GetBcd() * 10;
	if (date->mouth > 12)   date->mouth  = 12;
	if (date->mouth == 0)   date->mouth  = 1;
	date->year   = Rtc_GetBcd() + Rtc_GetBcd() * 10;
	Rtc_CE(0);
}

void Rtc_Put( struct DATE *date )
{
	Rtc_WR(1);
	Rtc_CE(1);
	_delay_us(1);
	
	Rtc_PutBcd( date->second % 10 );
	Rtc_PutBcd( date->second / 10 );
	Rtc_PutBcd( date->minute % 10 );
	Rtc_PutBcd( date->minute / 10 );
	Rtc_PutBcd( date->hour   % 10 );
	Rtc_PutBcd( date->hour   / 10 );
	Rtc_PutBcd( date->week );
	Rtc_PutBcd( date->day    % 10 );
	Rtc_PutBcd( date->day    / 10 );
	Rtc_PutBcd( date->mouth  % 10 );
	Rtc_PutBcd( date->mouth  / 10 );
	Rtc_PutBcd( date->year   % 10 );
	Rtc_PutBcd( date->year   / 10 );
	
	Rtc_CE( 0 );
	Rtc_WR( 0 );
}