//
//  UART Interface
//
//  ATmega88   2010.10.09
//
#include <avr/io.h>
#include <avr/interrupt.h>
#include "uart.h"

										// Buffer傫ƃXb^NȂȂ芄荞ݓȂ
#define RBUF_MAX	4					// Size of Receive buffer
#define SBUF_MAX	8					// Size of Send buffer

volatile unsigned char rbuf[RBUF_MAX];	// USART Receive buffer (ring buffer)
volatile uint8_t rwpnt;					// Receive buffer write pointer
volatile uint8_t rrpnt;					// Receive buffer read  pointer
volatile unsigned char sbuf[SBUF_MAX];	// USART Send buffer (ring buffer)
volatile uint8_t swpnt;					// Send buffer write pointer
volatile uint8_t srpnt;					// Send buffer read  pointer

//
// Initialize of UART
void UartInitialize(unsigned int baud)
{
	rwpnt = rrpnt = 0;
	swpnt = srpnt = 0;

	UBRR0 = 0x00;
										// UBRRݒ肷ƂɊۂߏĂB
										// Clock 8MHz CLKDIV8 : Off
	unsigned int ubrr = (8000000>>4)/baud-1;

	UBRR0 = ubrr;						// {[[g
	
	UCSR0A = 0b00000000;				// M,M荞݋
	UCSR0C = 0b00000110;				// stop bit 1bit, 8bitM, Non parity
	UCSR0B = (1 << RXEN0) | (1 << TXEN0) | (1 << RXCIE0) | (1 << TXCIE0);
}
//
// Check of Recive data
int UartIsReceived()
{
    return  (rwpnt !=  rrpnt) ? 1 : 0;
}
// Get of recive data
unsigned char UartGetReceivedData()
{
	unsigned char data;
	
    if ( UartIsReceived() ) {
  	 	data = rbuf[rrpnt++];
    	if (rrpnt >= RBUF_MAX) rrpnt = 0;
	}
	else data = (unsigned char)0;

    return data;
}
//
// Interrput of recive data
ISR(USART_RX_vect)
{
	uint8_t pnt;

	pnt = rwpnt;
    rbuf[rwpnt++] = UDR0;						// Set of receive data
    if (rwpnt >= RBUF_MAX) rwpnt = 0;
	if (rwpnt != rwpnt)    rwpnt = pnt;
}
//
// Internal module Send data
static void UartPrivateSendChar()
{
	if (swpnt != srpnt) UDR0 = sbuf[srpnt++];	// Send data
	if (srpnt >= SBUF_MAX) srpnt = 0;
}
//
// 荞݂ɂ鑗M
ISR(USART_TX_vect)
{
	UartPrivateSendChar();
}
//
// 1oCgM
void UartSendChar(unsigned char c)
{
	uint8_t pnt;

	// Full of send buffer
	pnt = swpnt;
	pnt += 1;
	if (pnt >= SBUF_MAX)  pnt = 0;
	while(pnt == srpnt);

	// Set of send buffer
	sbuf[swpnt] = c;
	swpnt = pnt;

	// Chek of send
	if (UCSR0A & (1<<UDRE0)) UartPrivateSendChar();
}
//
// Send of string
void UartSendString( char *p)
{
	while(*p) UartSendChar(*p++);
}


